package se.premex

import org.gradle.api.DefaultTask
import org.gradle.api.file.ConfigurableFileTree
import org.gradle.api.file.FileCollection
import org.gradle.api.tasks.CacheableTask
import org.gradle.api.tasks.Input
import org.gradle.api.tasks.InputFiles
import org.gradle.api.tasks.Nested
import org.gradle.api.tasks.OutputFile
import org.gradle.api.tasks.PathSensitive
import org.gradle.api.tasks.PathSensitivity
import org.gradle.api.tasks.TaskAction
import se.premex.toml.OwnershipFileResult
import java.io.File

@CacheableTask
open class GenerateOwnershipTask : DefaultTask() {

    @PathSensitive(PathSensitivity.RELATIVE)
    @InputFiles
    val ownershipFiles: FileCollection =
        project
            .fileTree(
                project.projectDir
            ) { files: ConfigurableFileTree ->
                files.include("**/OWNERSHIP.toml")
                    .exclude("**/build/**")
                    .exclude("**/.github/**")
                    .exclude("**/.bitbucket/**")
            }

    @Input
    val moduleRootDir = project.rootDir.path

    @Input
    val projectRootDir = project.rootProject.rootDir.path

    @OutputFile
    val gitownershipFile = project.file("build/generated/ownershipValidation/gitownership")

    @OutputFile
    val bitbucketownershipFile = project.file("build/generated/ownershipValidation/bitbucketownership")

    @Nested
    lateinit var ownershipExtension: OwnershipExtension

    @TaskAction
    fun validationTask() {
        clearFiles()

        ownershipFiles.sorted().forEach { ownershipFile ->

            var path = ownershipFile.relativeTo(File(projectRootDir)).path.replace("OWNERSHIP.toml", "")
            appendComment(path + "OWNERSHIP.toml")
            if (path == "") {
                path = "*"
            }

            val parsedOwnershipFile: OwnershipFileResult = TomlParser.parseFile(ownershipFile)

            appendLine(path + "\t" + parsedOwnershipFile.ownershipFile?.owner?.user!!)

            val owners: List<List<String>> = parsedOwnershipFile.ownershipFile.custom?.owners ?: listOf()
            if (owners.isNotEmpty()) {
                appendComment("Custom configurations")
            }
            owners.forEach {
                val key = it[0]
                val value = it[1]
                appendLine(key + "\t" + value)
            }
        }
        if (ownershipExtension.generateGithubOwners) {
            val target = File("$projectRootDir/.github/CODEOWNERS")
            if (target.exists()) {
                target.delete()
            }
            gitownershipFile.copyTo(target)
        }
        if (ownershipExtension.generateBitbucketOwners) {
            val target = File("$projectRootDir/.bitbucket/CODEOWNERS")
            if (target.exists()) {
                target.delete()
            }
            bitbucketownershipFile.copyTo(target)
        }
    }

    private fun clearFiles() {
        val message = "# This file is automatically generated by Premex ownership plugin\n" +
            "# Do not edit manually.\n" +
            "# https://plugins.gradle.org/plugin/se.premex.ownership\n" +
            "# https://github.com/premex-ab/ownership-gradle-plugin\n" +
            "\n"
        if (!gitownershipFile.exists()) {
            gitownershipFile.createNewFile()
            gitownershipFile.writeText(message)
        } else {
            gitownershipFile.writeText(message)
        }
        if (!bitbucketownershipFile.exists()) {
            bitbucketownershipFile.createNewFile()
            bitbucketownershipFile.writeText(message)
        } else {
            bitbucketownershipFile.writeText(message)
        }
    }

    private fun appendLine(txt: String) {
        gitownershipFile.appendText(txt + "\n")
        bitbucketownershipFile.appendText(txt + "\n")
    }

    private fun appendComment(txt: String) {
        appendLine("# $txt")
    }
}
